"""File containing constants, which are used for reading the data from the api JSON files"""
TICKER_NAME_FIELD = "ticker_name"
COMPANY_NAME_FIELD = "company_name"
CIK_FIELD = "cik"
DOCUMENT_AND_ENTITY_INFORMATION_FIELD = "document_and_entity_information"
IFRS_FULL_FIELD = "ifrs_full"
US_GAAP_FIELD = "us_gaap"
INVEST_FIELD = "invest"
IFRS_MINUS_FULL_API_FIELD = "ifrs-full"
SRT_FIELD = "srt"
US_MINUS_GAAP_API_FIELD = "us-gaap"
DEI_API_FIELD = "dei"
